#!/usr/bin/env python
#-*- coding: utf-8 -*-

from WebAppDIRAC.Lib.WebHandler import WebHandler, WErr
from DIRAC.Core.DISET.RPCClient import RPCClient
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getVOForGroup
import datetime

class TransferAppHandler(WebHandler):
    AUTH_PROPS = "authenticated"

    def __init__(self, *args, **kwargs ):
        super( TransferAppHandler, self ).__init__( *args, **kwargs )
        sessionData = self.getSessionData()
        self.user = sessionData['user'].get( 'username', '' )
        self.group = sessionData['user'].get( 'group', '' )
        self.vo = getVOForGroup( self.group )
        self.fc = FileCatalog( vo = self.vo )

        self.log.always(self.user)
        self.log.always(self.group)
        self.log.always(self.vo)
        self.log.always(self.fc)

    # = dataset management =
    # == list ==
    def web_datasetList(self):
        # use RPCClient
        transferRequest = RPCClient("Transfer/Dataset")
        condDict = {}
        orderby = []
        start = 0
        limit = 50
        res = transferRequest.showtotal(condDict)
        #self.log.always(res)
        #  {'OK': True, 'rpcStub': (('Transfer/Dataset', {'skipCACheck': False, 'keepAliveLapse': 150, 'delegatedGroup': 'bes_user', 'delegatedDN': '/C=CN/O=HEP/OU=PHYS/O=IHEP/CN=Tian Yan', 'timeout': 600}), 'showtotal', ({},)), 'Value': 68L} 
        limit = res["Value"]

        res = transferRequest.show(condDict, orderby, start, limit)
        #self.log.always(res)
        # {'OK': True, 
        #  'rpcStub': (('Transfer/Dataset', {'delegatedDN': '/C=CN/O=HEP/OU=PHYS/O=IHEP/CN=Tian Yan', 'timeout': 600, 'skipCACheck': False, 'keepAliveLapse': 150, 'delegatedGroup': 'bes_user'}), 'show', ({}, [], 0, 68L)), 
        #  'Value': (
        #            (1L, 'my-dataset', 'lintao'), 
        #            (2L, 'jpsi-test', 'besdirac02.ihep.ac.cn'), 
        #            (3L, 'jpsi-test-10', 'besdirac02.ihep.ac.cn'), 
        #           )
        # } 
        data = []
        if res["OK"]:
            for id, dataset, owner in res["Value"]:
                data.append({
                    "id": id,
                    "owner": owner, 
                    "dataset": dataset,
                })
        self.write({"result": data})
    def web_datasetListFiles(self):
        self.log.debug(self.request.arguments)
        dataset = None
        if self.request.arguments.get("dataset", None):
            dataset = self.request.arguments["dataset"][0]
        data = []
        cache = []
        if dataset:
            RPC = RPCClient("Transfer/Dataset")
            res = RPC.list(dataset)
            #self.log.always(res)
            # {'OK': True, 
            #  'rpcStub': (('Transfer/Dataset', {'skipCACheck': False, 'keepAliveLapse': 150, 'delega tedGroup': 'bes_user', 'delegatedDN': '/C=CN/O=HEP/OU=PHYS/O=IHEP/CN=Tian Yan', 'timeout': 600}), 'list', ('jpsi-all-ok',)), 
            #  'Value': [
            #   (176L, '/zhanggang_test/File/jpsi/6.6.4/mc/inclusive/round02/stream001/jpsi2009_stream001_run10005_file14', 7L), 
            #   (177L, '/zhanggang_test/File/jpsi/6.6.4/mc/inclusive /round02/stream001/jpsi2009_stream001_run10137_file7', 7L), 
            #  ]
            # }
            if res["OK"]:
                cache = res["Value"]


        for i,f,di in cache:
            data.append({
                "id": i,
                "file": f,
            })
        self.write({"result": data})
    # == create ==
    # == delete ==

    # == list in DFC ==
    def web_DFCdatasetList(self):
        # Load DFC
        datasetname = {"":""}
        res = self.fc.getDatasets(datasetname)
        self.log.always(res)
        #if not res["OK"]:
        #    self.log.always( "\n".join(res["CallStack"]) )
        #res = self.fc.listDirectory("/")
        #self.log.always(res)
        
        # use RPCClient
        transferRequest = RPCClient("Transfer/Dataset")
        condDict = {}
        orderby = []
        start = 0
        limit = 50
        res = transferRequest.showtotal(condDict)
        #self.log.always(res)
        #  {'OK': True, 'rpcStub': (('Transfer/Dataset', {'skipCACheck': False, 'keepAliveLapse': 150, 'delegatedGroup': 'bes_user', 'delegatedDN': '/C=CN/O=HEP/OU=PHYS/O=IHEP/CN=Tian Yan', 'timeout': 600}), 'showtotal', ({},)), 'Value': 68L} 
        limit = res["Value"]

        res = transferRequest.show(condDict, orderby, start, limit)
        #self.log.always(res)
        # {'OK': True, 
        #  'rpcStub': (('Transfer/Dataset', {'delegatedDN': '/C=CN/O=HEP/OU=PHYS/O=IHEP/CN=Tian Yan', 'timeout': 600, 'skipCACheck': False, 'keepAliveLapse': 150, 'delegatedGroup': 'bes_user'}), 'show', ({}, [], 0, 68L)), 
        #  'Value': (
        #            (1L, 'my-dataset', 'lintao'), 
        #            (2L, 'jpsi-test', 'besdirac02.ihep.ac.cn'), 
        #            (3L, 'jpsi-test-10', 'besdirac02.ihep.ac.cn'), 
        #           )
        # } 
        data = []
        if res["OK"]:
            for id, dataset, owner in res["Value"]:
                data.append({
                    "id": id,
                    "owner": owner, 
                    "dataset": dataset,
                })
        self.write({"result": data})

    # == list in DFC ==
    def web_DFCdatasetListFiles(self):
        dataset = None
        if self.request.arguments.get("dataset", None):
            dataset = self.request.arguments["dataset"][0]
        data = []
        cache = []
        if dataset:
            # TODO
            res = self.fc.getDatasetFiles(dataset)
            self.log.always(res)


        for i,f,di in cache:
            data.append({
                "id": i,
                "file": f,
            })
        self.write({"result": data})

    # = transfer request (not including file list) =
    # == list ==
    def web_requestList(self):
        # create dummy data
        data = []
        # use RPCClient
        RPC = RPCClient("Transfer/TransferRequest")
        cond = {}
        res = RPC.statustotal(cond)
        #self.log.always(res)
        # {'OK': True, 
        #  'rpcStub': (('Transfer/TransferRequest', {'skipCACheck': False, 'keepAliveLapse': 150, 'delegatedGroup': 'bes_user', 'delegatedDN': '/C=CN/O=HEP/OU=PHYS/O=IHEP/CN=Tian Yan', 'timeout': 600}), 'statustotal', ({},)),
        #  'Value': 159L}
        start = 0
        limit = res["Value"]
        orderby = ["id:DESC"]
        cache = []
        res = RPC.statuslimit(cond, orderby, start, limit)
        if res["OK"]:
            cache = res["Value"]
        #self.log.always(cache)
        # (159L, 
        #  'yant', 
        #  'prod_rtg_r04_add_150104', 
        #  'IHEPD-USER', 
        #  'WHU-USER', 
        #  'DIRACDMS', 
        #  datetime.datetime(2015, 6, 12, 8, 41, 19), 
        #  'finish'),

        for vv in cache:
            data.append({
                "id": vv[0],
                "owner": vv[1], 
                "dataset": vv[2],
                "srcSE": vv[3],
                "dstSE": vv[4],
                "protocol": vv[5], 
                "submitTime": vv[6].strftime("%Y-%m-%d %H:%M [UTC]") if vv[6] else "",
                "status": vv[7],
            })
        self.write({"result": data})

    # == new ==
    # create a new transfer request
    def web_requestNew(self):
        # validate
        # * dataset
        # * srcse
        # * dstse
        # * protocol
        self.log.debug(self.request.arguments)
        valid_list = ["dataset", "srcse", "dstse", "protocol"]
        build_input_param = {}
        for k in valid_list:
            if not self.request.arguments.has_key(k):
                raise WErr( 400, "Missing %s" % k )
            build_input_param[k] = self.request.arguments[k][0]
        self.log.debug(build_input_param)
        # check the data
        ## SE
        if build_input_param["dstse"] == build_input_param["srcse"]:
            raise WErr( 400, "dstse and srcse are same" )
        ## protocol
        if build_input_param["protocol"] not in ["DIRACDMS", "DIRACFTS"]:
            raise WErr( 400, "protocol %s is wrong"%build_input_param["protocol"] )
        # create
        RPC = RPCClient("Transfer/TransferRequest")
        res = RPC.create(build_input_param["dataset"],
                         build_input_param["srcse"],
                         build_input_param["dstse"],
                         build_input_param["protocol"])
        # TODO how to return error to the user?
        if not res["OK"]:
            self.log.error(res)

        self.set_status(200)
        self.finish()
        self.log.debug("finish")

    # = file list of one request =
    # == list ==
    def web_requestListFiles(self):
        self.log.debug(self.request.arguments)
        cache = []
        if self.request.arguments.get("reqid", None):
            reqid = int(self.request.arguments["reqid"][0])
            RPC = RPCClient("Transfer/TransferRequest")
            cond = {"trans_req_id": reqid}
            res = RPC.show(cond)
            if res["OK"]:
                cache = res["Value"]
        #self.log.always(cache)
        # (1L, 
        #  '/path/does/not/exist', 
        #  1L, # datasetid
        #  datetime.datetime(2013, 8, 23, 3, 12, 37), 
        #  datetime.datetime(2013, 8, 23, 3, 14, 37), 
        #  'finish', 
        #  'error'
        # )
        data = []
        for vv in cache:
            data.append({
                "id": vv[0],
                "LFN": vv[1],
                "starttime": vv[3].strftime("%Y-%m-%d %H:%M [UTC]") if vv[3] else "",
                "finishtime": vv[4].strftime("%Y-%m-%d %H:%M [UTC]") if vv[4] else "",
                "status": vv[5],
                "error": vv[6],
            })
        self.write({"result": data})





from datetime import datetime, timedelta
from DIRAC import S_OK, S_ERROR
from DIRAC.ResourceStatusSystem.Command.Command                 import Command
from IHEPDIRAC.ResourceStatusSystem.Client.ResourceManagementIHEPClient import ResourceManagementIHEPClient

__RCSID__ = '$Id:  $'

class SAMIHEPCommand( Command ):

  def __init__( self, args = None, clients = None ):

    super( SAMIHEPCommand, self ).__init__( args, clients )

    if 'ResourceManagementIHEPClient' in self.apis:
      self.rmIHEPClient = self.apis[ 'ResourceManagementIHEPClient' ]
    else:
      self.rmIHEPClient = ResourceManagementClient()


  def doCommand( self ):
    element = self.args[ 'element' ]
    elementName = self.args[ 'name' ]

    lastCheckTime = datetime.utcnow().replace(microsecond = 0) - timedelta(hours = 24)

    if element == 'Site':
      queryRes = self.rmIHEPClient.selectSiteSAMStatus( site = elementName,
                                                    meta = { 'newer' : [ 'LastCheckTime', lastCheckTime ] } )
    elif element == 'Resource':
      queryRes = self.rmIHEPClient.selectResourceSAMStatus( elementName = elementName,
                                                        meta = { 'newer' : [ 'LastCheckTime', lastCheckTime ] } )
    else:
      return self.returnERROR( S_ERROR('No SAM information for %s element' % element ) )

    if not queryRes[ 'OK' ]:
      return self.returnERROR( queryRes )
    records = queryRes[ 'Value' ]
    columns = queryRes[ 'Columns' ]

    results = []
    for record in records:
      results.append( dict( zip( columns, record ) ) )
    return S_OK( results )




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# author: linlei

import re
import os.path


def judgeType(dstfile):
    if os.path.exists(dstfile):    
        #judge whether type of this dst file is "all"
        all = re.compile(r"run_\d+_All_file\d+_SFO-\d+.")
        #judge whether type of this dst file is "others"
        #others = re.compile(r"[A-Za-z]+_[A-Za-z]+\d*_stream\d+_\d+_\d+.*")
        others = re.compile(r"\w+_stream\d+_run\d+_file\d+.*")
       
        flag_all = all.search(dstfile)
        flag_others = others.search(dstfile)

        if flag_all is not None:
            #return flag_all.group()
            return "all"
        elif flag_others is not None:
            #return flag_others.group()
            return "others"
        else:
            print "can't check out this file's type"
            return  
    

    




from types import DictType, FloatType, IntType, ListType, LongType, StringTypes, TupleType, UnicodeType

import time
import json

# DIRAC
from DIRAC                                import gConfig, gLogger, S_ERROR, S_OK
from DIRAC.Core.DISET.RequestHandler      import RequestHandler, getServiceOption
from DIRAC.Core.DISET.RPCClient           import RPCClient
from DIRAC.Core.Security                  import Properties

from DIRAC.WorkloadManagementSystem.DB.JobDB      import JobDB
from IHEPDIRAC.WorkloadManagementSystem.DB.TaskDB  import TaskDB

__RCSID__ = '$Id: $'

# This is a global instance of the TaskDB class
gTaskDB = None
gJobDB = None

def initializeTaskManagerHandler( serviceInfo ):

  global gTaskDB, gJobDB

  gTaskDB = TaskDB()
  gJobDB = JobDB()

  return S_OK()

class TaskManagerHandler( RequestHandler ):

  def initialize( self ):
    credDict = self.getRemoteCredentials()
    self.owner          = credDict[ 'username' ]
    self.ownerDN        = credDict[ 'DN' ]
    self.ownerGroup     = credDict[ 'group' ]
    self.userProperties = credDict[ 'properties' ]


################################################################################
# exported interfaces

  types_createTask = [ StringTypes, DictType ]
  def export_createTask( self, taskName, taskInfo, jobInfos ):
    """ Create a new task
    """
    if Properties.NORMAL_USER not in self.userProperties and Properties.JOB_ADMINISTRATOR not in self.userProperties:
      return S_ERROR( 'Access denied to create task' )

    jobIDs = self.__filterJobAccess( jobInfos.keys() )

    status = 'Init'
    result = gTaskDB.createTask( taskName, status, self.owner, self.ownerDN, self.ownerGroup, taskInfo )
    if not result['OK']:
      return result
    taskID = result['Value']

    result = gTaskDB.insertTaskHistory( taskID, status, 'New task created' )
    if not result['OK']:
      return result

    for jobID, jobInfo in jobInfos.items():
      if jobID in jobIDs:
        result = gTaskDB.addTaskJob( taskID, jobID, jobInfo )
        if not result['OK']:
          return result

    result = gTaskDB.updateTaskStatus( taskID, 'Ready', 'Task is ready for processing' )
    if not result['OK']:
      return result

    return S_OK( taskID )

#  types_activateTask = [ [IntType, LongType] ]
#  def export_activateTask( self, taskID ):
#    """ Activate the task
#    """
#    if not self.__hasTaskAccess( taskID ):
#      return S_ERROR( 'Access denied to activate task %s' % taskID )
#
#    result = gTaskDB.getTaskStatus( taskID )
#    if not result['OK']:
#      return result
#    status = result['Value']
#
#    if status != 'Init':
#      self.log.error( 'Can only activate task with "Init" status: task %s' % taskID )
#      return S_ERROR( 'Can only activate task with "Init" status: task %s' % taskID )
#
#    return gTaskDB.updateTaskStatus( taskID, 'Ready', 'Task is activated' )

  types_removeTask = [ [IntType, LongType] ]
  def export_removeTask( self, taskID ):
    """ Delete the task
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to remove task %s' % taskID )

    return gTaskDB.updateTaskStatus( taskID, 'Removed', 'Task is removed' )

#  types_addTaskJob = [ [IntType, LongType], [IntType, LongType], DictType ]
#  def export_addTaskJob( self, taskID, jobID, jobInfo ):
#    """ Add a job to the task
#    """
#    if not self.__hasTaskAccess( taskID ):
#      return S_ERROR( 'Access denied for task %s: Can not add job %s to task' % ( taskID, jobID ) )
#    if not self.__hasJobAccess( jobID ):
#      return S_ERROR( 'Access denied for job %s: Can not add job to task %s' % ( jobID, taskID ) )
#
#    result = gTaskDB.getTaskStatus( taskID )
#    if not result['OK']:
#      return result
#    status = result['Value']
#
#    if status != 'Init':
#      self.log.error( 'Can only add job to "Init" status: task %s' % taskID )
#      return S_ERROR( 'Can only add job to "Init" status: task %s' % taskID )
#
#    return gTaskDB.addTaskJob( taskID, jobID, jobInfo )
#
#  types_updateTaskInfo = [ [IntType, LongType], DictType ]
#  def export_updateTaskInfo( self, taskID, taskInfo ):
#    """ Update the task info
#    """
#    if not self.__hasTaskAccess( taskID ):
#      return S_ERROR( 'Access denied to update task info for task %s' % taskID )
#
#    result = gTaskDB.getTaskInfo( taskID )
#    if not result['OK']:
#      return result
#
#    newTaskInfo = result['Value']
#    newTaskInfo.update( taskInfo )
#
#    return gTaskDB.updateTaskInfo( taskID, newTaskInfo )

  types_renameTask = [ [IntType, LongType], StringTypes ]
  def export_renameTask( self, taskID, newName ):
    """ Rename the task
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to rename task %s' % taskID )

    return gTaskDB.renameTask( taskID, newName )

################################################################################

  types_getTaskOwners = []
  def export_getTaskOwners( self ):
    """ Return Distinct Values of Owner
    """
    return gTaskDB.getDistinctTaskAttributes( 'Owner' )

  types_getTaskOwnerGroups = []
  def export_getTaskOwnerGroups( self ):
    """ Return Distinct Values of OwnerGroup
    """
    return gTaskDB.getDistinctTaskAttributes( 'OwnerGroup' )

  types_getTaskCount = [ DictType ]
  def export_getTaskCount( self, condDict ):
    """ Get task count
    """
    if Properties.NORMAL_USER not in self.userProperties and Properties.JOB_ADMINISTRATOR not in self.userProperties:
      return S_ERROR( 'Access denied to get task count' )

    if Properties.NORMAL_USER in self.userProperties:
      condDict['OwnerDN'] = self.ownerDN
      condDict['OwnerGroup'] = self.ownerGroup

      if 'Status' not in condDict:
        condDict['Status'] = [ 'Init', 'Ready', 'Processing', 'Finished', 'Expired' ]
      else:
        condDict['Status'] = list( condDict['Status'] )
        condDict['Status'] = [ v for v in condDict['Status'] if v != 'Removed' ]

    return gTaskDB.getTaskCount( condDict )

  types_getTasks = [ DictType, [IntType, LongType], [IntType, LongType], StringTypes, [IntType, LongType] ]
  def export_getTasks( self, condDict, limit, offset, orderAttribute, realTimeProgress ):
    """ Get task list
    """
    if Properties.NORMAL_USER not in self.userProperties and Properties.JOB_ADMINISTRATOR not in self.userProperties:
      return S_ERROR( 'Access denied to get tasks' )

    if Properties.NORMAL_USER in self.userProperties:
      condDict['OwnerDN'] = self.ownerDN
      condDict['OwnerGroup'] = self.ownerGroup

      if 'Status' not in condDict:
        condDict['Status'] = [ 'Init', 'Ready', 'Processing', 'Finished', 'Expired' ]
      else:
        condDict['Status'] = list( condDict['Status'] )
        condDict['Status'] = [ v for v in condDict['Status'] if v != 'Removed' ]

    if limit < 0:
      limit = False
    outFields = ['TaskID', 'TaskName', 'Status', 'Owner', 'OwnerDN', 'OwnerGroup', 'CreationTime', 'UpdateTime', 'JobGroup', 'Site', 'Progress']
    result = gTaskDB.getTasks( outFields, condDict, limit, offset, orderAttribute )
    if not result['OK']:
      self.log.error( result['Message'] )
      return S_ERROR( result['Message'] )

    tasks = []
    for outValues in result['Value']:
      progress = {}
      if realTimeProgress and outValues[2] in ['Init', 'Ready', 'Processing', 'Finished']:
        result = self.__getTaskProgress( outValues[0] )
        if not result['OK']:
          self.log.error( result['Message'] )
          return result
        progress = result['Value']
      tasks.append( self.__generateTaskResult( outFields, outValues, progress ) )

    return S_OK( tasks )

  types_getTask = [ [IntType, LongType], [IntType, LongType] ]
  def export_getTask( self, taskID, realTimeProgress ):
    """ Get task
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to get task for task %s' % taskID )

    outFields = ['TaskID', 'TaskName', 'Status', 'Owner', 'OwnerDN', 'OwnerGroup', 'CreationTime', 'UpdateTime', 'JobGroup', 'Site', 'Info']
    if not realTimeProgress:
      outFields.append( 'Progress' )
    result = gTaskDB.getTask( taskID, outFields )
    if not result['OK']:
      self.log.error( result['Message'] )
      return S_ERROR( result['Message'] )
    outValues = result['Value']

    progress = {}
    if realTimeProgress:
      result = self.__getTaskProgress( taskID )
      if not result['OK']:
        self.log.error( result['Message'] )
        return result
      progress = result['Value']
    return S_OK( self.__generateTaskResult( outFields, outValues, progress ) )

  types_getTaskStatus = [ [IntType, LongType] ]
  def export_getTaskStatus( self, taskID ):
    """ Get task status
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to get task status for task %s' % taskID )

    return gTaskDB.getTaskStatus( taskID )

  types_getTaskInfo = [ [IntType, LongType] ]
  def export_getTaskInfo( self, taskID ):
    """ Get task info
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to get task info for task %s' % taskID )

    return gTaskDB.getTaskInfo( taskID )

  types_getTaskProgress = [ [IntType, LongType] ]
  def export_getTaskProgress( self, taskID ):
    """ Get task progress
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to get task progress for task %s' % taskID )

    result = self.__getTaskProgress( taskID )
    if not result['OK']:
      return result
    progress = result['Value']
    self.log.debug( 'Task %d Progress: %s' % ( taskID, progress ) )
    result = gTaskDB.updateTaskProgress( taskID, progress )
    if not result['OK']:
      return result
    return S_OK( progress )

  types_getTaskHistories = [ [IntType, LongType] ]
  def export_getTaskHistories( self, taskID ):
    """ Get task histories
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to get task histories for task %s' % taskID )

    return gTaskDB.getTaskHistories( taskID )

  types_getTaskJobs = [ [IntType, LongType] ]
  def export_getTaskJobs( self, taskID ):
    """ Get task jobs
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to get task jobs for task %s' % taskID )

    return gTaskDB.getTaskJobs( taskID )

  types_getJobs = [ ListType, ListType ]
  def export_getJobs( self, jobIDs, outFields ):
    """ Get jobs
    """
    jobIDs = self.__filterJobOfTaskAccess( jobIDs )

    return gTaskDB.getJobs( jobIDs, outFields )

  types_getTaskIDFromJob = [ [IntType, LongType] ]
  def export_getTaskIDFromJob( self, jobID ):
    """ Get task ID from job ID
    """
    result = gTaskDB.getTaskIDFromJob( jobID )
    if not result['OK']:
      return result

    return S_OK( self.__filterTaskAccess( result['Value'] ) )

  types_getJobInfo = [ [IntType, LongType] ]
  def export_getJobInfo( self, jobID ):
    """ Get job info
    """
    if not self.__hasJobForTaskAccess( jobID ):
      return S_ERROR( 'Access denied to get job info for job %s' % jobID )

    return gTaskDB.getJobInfo( jobID )


  types_getTaskJobsStatistics = [ [IntType, LongType], StringTypes ]
  def export_getTaskJobsStatistics( self, taskID, statusType ):
    """ Get statistics of all jobs in the task
    """
    if not self.__hasTaskAccess( taskID ):
      return S_ERROR( 'Access denied to get task jobs %s statistics for task %s' % ( statusType, taskID ) )

    return self.__getTaskStatusCount( taskID, statusType )


################################################################################
# private functions

  def __isSameDNGroupForTask( self, taskID ):
    result = gTaskDB.getTask( taskID, ['OwnerDN', 'OwnerGroup'] )
    if not result['OK']:
      self.log.error( result['Message'] )
      return False
    if not result['Value']:
      self.log.error( 'Task ID %s not found' % taskID )
      return False

    taskOwnerDN, taskOwnerGroup = result['Value']
    if self.ownerDN == taskOwnerDN and self.ownerGroup == taskOwnerGroup:
      return True

    return False

  def __hasTaskAccess( self, taskID ):
    if Properties.JOB_ADMINISTRATOR in self.userProperties:
      return True

    if Properties.NORMAL_USER in self.userProperties:
      if self.__isSameDNGroupForTask( taskID ):
        result = gTaskDB.getTaskStatus( taskID )
        if result['OK'] and result['Value'] != 'Removed':
          return True

    return False


  def __isSameDNGroupForJob( self, jobID ):
    result = gJobDB.getAttributesForJobList( [jobID], ['OwnerDN', 'OwnerGroup'] )
    if not result['OK']:
      self.log.error( result['Message'] )
      return False
    if not result['Value']:
      self.log.error( 'Job %s not found' % jobID )
      return False
    jobOwnerDN = result['Value'][jobID]['OwnerDN']
    jobOwnerGroup = result['Value'][jobID]['OwnerGroup']

    if self.ownerDN == jobOwnerDN and self.ownerGroup == jobOwnerGroup:
      return True

    return False

  def __hasJobAccess( self, jobID ):
    if Properties.JOB_ADMINISTRATOR in self.userProperties:
      return True

    if Properties.NORMAL_USER in self.userProperties:
      if self.__isSameDNGroupForJob( jobID ):
        return True

    return False

  def __hasJobForTaskAccess( self, jobID ):
    result = gTaskDB.getTaskIDFromJob( jobID )
    if not result['OK']:
      return result

    taskIDs = self.__filterTaskAccess( result['Value'] )

    for taskID in taskIDs:
      if self.__hasTaskAccess( taskID ):
        return True

    return False


  def __filterSameDNGroupForTask( self, taskIDs ):
    result = gTaskDB.getAttributesForTaskList( taskIDs, ['OwnerDN', 'OwnerGroup'] )
    if not result['OK']:
      self.log.error( result['Message'] )
      return []
    if not result['Value']:
      self.log.error( 'Task %s not found' % taskIDs )
      return []

    return [ taskID for taskID in result['Value'].keys() if self.ownerDN == result['Value'][taskID]['OwnerDN'] and self.ownerGroup == result['Value'][taskID]['OwnerGroup'] ]

  def __filterTaskAccess( self, taskIDs ):
    if Properties.JOB_ADMINISTRATOR in self.userProperties:
      return taskIDs

    if Properties.NORMAL_USER in self.userProperties:
      return self.__filterSameDNGroupForTask( taskIDs )

    return []


  def __filterSameDNGroupForJob( self, jobIDs ):
    result = gJobDB.getAttributesForJobList( jobIDs, ['OwnerDN', 'OwnerGroup'] )
    if not result['OK']:
      self.log.error( result['Message'] )
      return []
    if not result['Value']:
      self.log.error( 'Job %s not found' % jobIDs )
      return []

    return [ jobID for jobID in result['Value'].keys() if self.ownerDN == result['Value'][jobID]['OwnerDN'] and self.ownerGroup == result['Value'][jobID]['OwnerGroup'] ]

  def __filterJobAccess( self, jobIDs ):
    if Properties.JOB_ADMINISTRATOR in self.userProperties:
      return jobIDs

    if Properties.NORMAL_USER in self.userProperties:
      return self.__filterSameDNGroupForJob( jobIDs )

    return []


  def __filterJobOfTaskAccess( self, jobIDs ):
    """ Only filter jobs with right access to the related Task
    """
    result = gTaskDB.getJobs( jobIDs, ['JobID', 'TaskID'] )
    if not result['OK']:
      self.log.error( result['Message'] )
      return []
    taskIDs = list( set( [ line[1] for line in result['Value'] ] ) )

    newTaskIDs = self.__filterTaskAccess( taskIDs )

    return [ line[0] for line in result['Value'] if line[1] in newTaskIDs ]


################################################################################

  def __getTaskJobAttributes( self, taskID, outFields ):
    result = gTaskDB.getTaskJobs( taskID )
    if not result['OK']:
      return result
    jobIDs = result['Value']

    return gJobDB.getAttributesForJobList( jobIDs, outFields )

  def __getTaskStatusCount( self, taskID, statusType ):
    result = self.__getTaskJobAttributes( taskID, [statusType] )
    if not result['OK']:
      return result
    statuses = result['Value']

    statusCount = {}
    for jobID in statuses:
      status = statuses[jobID][statusType]
      if status not in statusCount:
        statusCount[status] = 0
      statusCount[status] += 1

    return S_OK( statusCount )

  def __getTaskProgress( self, taskID ):
    result = gTaskDB.getTaskJobs( taskID )
    if not result['OK']:
      return result
    jobIDs = result['Value']

    result = gJobDB.getAttributesForJobList( jobIDs, ['Status'] )
    if not result['OK']:
      return result
    statuses = result['Value']

    progress = { 'Total': 0, 'Done': 0, 'Failed': 0, 'Running': 0, 'Waiting': 0, 'Deleted': 0 }
    progress['Total'] = len(jobIDs)
    for jobID in jobIDs:
      if jobID in statuses:
        status = statuses[jobID]['Status']
        if status in ['Done']:
          progress['Done'] += 1
        elif status in ['Failed', 'Stalled', 'Killed']:
          progress['Failed'] += 1
        elif status in ['Running', 'Completed']:
          progress['Running'] += 1
        else:
          progress['Waiting'] += 1
      else:
        progress['Deleted'] += 1

    return S_OK( progress )


################################################################################

  def __generateTaskResult( self, outFields, outValues, progress ):
    taskResult = {}
    for k,v in zip(outFields, outValues):
      if k in ['Progress', 'Info']:
        taskResult[k] = json.loads( v )
      else:
        taskResult[k] = v

    if progress:
      taskResult['Progress'] = progress

    return taskResult



# $HeadURL $
""" PublisherHandler

  PublisherHandler for IHEPDIRAC

"""

import  math
from datetime import datetime, timedelta
from types    import NoneType

# DIRAC
from DIRAC                                                      import gLogger, S_OK, gConfig, S_ERROR
from DIRAC.Core.DISET.RequestHandler                            import RequestHandler
from IHEPDIRAC.ResourceStatusSystem.Utilities                   import CSHelpers
from DIRAC.ConfigurationSystem.Client.Helpers import Registry
from DIRAC.ResourceStatusSystem.Client.ResourceStatusClient import ResourceStatusClient
from IHEPDIRAC.ResourceStatusSystem.Client.ResourceManagementIHEPClient import ResourceManagementIHEPClient
from IHEPDIRAC.ResourceStatusSystem.Utilities import BESUtils

__RCSID__ = '$Id: $'

# RSS Clients
rsClient = None
rmClient = None

def initializePublisherIHEPHandler(_serviceInfo):
  """
  Handler initialization in the usual horrible way.
  """

  global rsClient
  rsClient = ResourceStatusClient()

  global rmClient
  rmClient = ResourceManagementIHEPClient()

  return S_OK()

class PublisherIHEPHandler( RequestHandler ):
  """
  PublisherHandler inherits from DIRAC's PublisherHandler.
  """

  def __init__(self, *args, **kwargs):
    """
    Constructor
    """
    super(PublisherIHEPHandler, self).__init__(*args, **kwargs)

  # My Methods ...................................................

  types_getVOs = []
  def export_getVOs(self):
    """
    Returns the list of all VOs.
    """

    gLogger.info('getVOs')

    return Registry.getVOs()

  types_getVOByGroup = [ str ]
  def export_getVOByGroup(self, group):
    """
    Returns the vo which the group belongs to.
    """

    gLogger.info('getVOByGroup')

    return S_OK(Registry.getVOMSVOForGroup(group))

  types_getDomains = []
  def export_getDomains(self):
    """
    Returns the list of all site domains.
    """

    gLogger.info('getDomains')

    return gConfig.getSections('Resources/Sites')

  types_getDomainSites = [ str ]
  def export_getDomainSites(self, domain):
    """
    Returns the list of sites under the specified domain.
    """

    gLogger.info('getDomainSites')

    return gConfig.getSections('Resources/Sites/' + domain)

  types_getComputingElements = []
  def export_getComputingElements(self):
    """
    Returns the list of all CEs.
    """

    gLogger.info('getComputingElements')

    return CSHelpers.getComputingElements()

  types_getStorageElements = []
  def export_getStorageElements(self):
    """
    Returns the list of all SEs.
    """

    gLogger.info('getStorageElements')

    return CSHelpers.getStorageElements()

  types_getSites = []
  def export_getSites( self ):
    """
    Returns list of all sites considered by RSS

    :return: S_OK( [ sites ] ) | S_ERROR
    """

    gLogger.info( 'getSites' )
    return CSHelpers.getSites()

  types_getSiteVO = [ str ]
  def export_getSiteVO(self, siteName):
    """
    Returns the VO for the given site.
    """

    gLogger.info('getSiteVO')

    vos = BESUtils.getSiteVO( siteName )
    return S_OK( vos )

  types_getSiteResource = [ str ]
  def export_getSiteResource(self, siteName):
    """
    Returns the dictionary with CEs and SEs for the given site.

    :return: S_OK( { 'ComputingElement' : celist, 'StorageElement' : selist } ) | S_ERROR
    """

    gLogger.info('getSiteResource')

    siteType = siteName.split('.')[ 0 ]

    if siteType == 'CLOUD':
      ces = []
    else:
      ces = CSHelpers.getSiteComputingElements(siteName)

    ses = CSHelpers.getSiteStorageElements(siteName)

    return S_OK({ 'ComputingElement' : ces, 'StorageElement' : ses })

  types_getSitesSAMSummary = [  (str, NoneType, list), (str, NoneType, list) ]
  def export_getSitesSAMSummary(self, sitesName, vo = None):
    """
    Return the dictionary with SAM summary information for the given sites.

    :return: S_OK( { site : { 'CEStatus' : 'OK'
                                                   'SEStatus' : 'Bad'
                                                   } } ) | S_ERROR
    """

    gLogger.info('getSitesSAMSummary')

    sitesSAMSummary = {}

    vo = vo or 'all'
    queryRes = rmClient.selectSiteSAMStatus(site = sitesName, vO = vo,
                                            meta = { 'newer' : [ 'LastCheckTime',
                                                                datetime.utcnow().replace(microsecond = 0) - timedelta(hours = 24) ] })
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]
    columns = queryRes[ 'Columns' ]

    for record in records:
      recordDict = dict(zip(columns, record))
      siteName = recordDict[ 'Site' ]
      sitesSAMSummary[ siteName ] = { 'CEStatus' : recordDict[ 'CEStatus' ], 'SEStatus' : recordDict[ 'SEStatus' ] }

    return S_OK(sitesSAMSummary)

  types_getSitesStorageSummary = [ (str, NoneType, list) ]
  def export_getSitesStorageSummary(self, sitesName):
    """
    Return the dictionary with storage summary information for the given sites.

    :return: S_OK( { site : { 'MaxStorage' :  1000.00
                                                   'FreeStorage' :  200.00
                                                   'StorageUsage' : 80.0
                                                   } } ) | S_ERROR
    """

    gLogger.info('getSitesStorageSummary')

    sitesStorageSummary = {}

    ses = set()
    for siteName in sitesName:
      se = CSHelpers.getSiteStorageElements(siteName)
      if se:
        sitesStorageSummary[ siteName ] = se[ 0 ]
        ses.add(se[ 0 ])

    if len(ses) == 0:
      return S_OK(sitesStorageSummary)

    queryRes = rmClient.selectStorageCache(sE = list(ses))
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]
    columns = queryRes[ 'Columns' ]

    seInfo = {}
    for record in records:
      recordDict = dict(zip(columns, record))
      seName = recordDict.pop('SE')
      seDict = {}
      seDict[ 'MaxStorage' ] = math.floor(float(recordDict[ 'Occupied' ] + recordDict[ 'Free' ]) / 1024 / 1024 / 1024 * 100) / 100
      seDict[ 'FreeStorage' ] = math.floor(float(recordDict[ 'Free' ]) / 1024 / 1024 / 1024 * 100) / 100
      seDict[ 'StorageUsage' ] = recordDict[ 'Usage' ]
      seInfo[ seName ] = seDict

    for siteName, seName in sitesStorageSummary.items():
      sitesStorageSummary[ siteName ] = seInfo[ seName ]

    return S_OK(sitesStorageSummary)

  types_getSitesJobSummary = [ (str, NoneType, list) ]
  def export_getSitesJobSummary(self, sitesName):
    """
    Return the dictionary with job summary information for the given sites.

    :return: S_OK( { site : { 'Running' :  222
                                                   'Waiting' :  222
                                                   'Done' : 222
                                                   'Failed' : 222
                                                   'Efficiency' : 50.0
                                                   'MaxJobs' : 666
                                                   'JobUsage' : 33.3
                                                   } } ) | S_ERROR
    """

    gLogger.info('getSitesJobSummary')

    sitesJobSummary = {}

    queryRes = rmClient.selectJobCache(site = sitesName)
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]
    columns = queryRes[ 'Columns' ]

    for record in records:
      recordDict = dict(zip(columns, record))
      siteName = recordDict[ 'Site' ]
      jobDict = {}
      jobDict[ 'Running' ] = recordDict[ 'Running' ]
      jobDict[ 'Waiting' ] = recordDict[ 'Waiting' ]
      jobDict[ 'Done' ] = recordDict[ 'Done' ]
      jobDict[ 'Failed' ] = recordDict[ 'Failed' ]
      jobDict[ 'Efficiency' ] = recordDict[ 'Efficiency' ]
      sitesJobSummary[ siteName ] = jobDict

    siteMaxJobs = self.__getSitesMaxJobs(sitesName)
    if not siteMaxJobs[ 'OK' ]:
      return siteMaxJobs
    siteMaxJobs = siteMaxJobs[ 'Value' ]

    for siteName, jobDict in sitesJobSummary.items():
      maxJobs =  siteMaxJobs.get(siteName)
      if maxJobs:
        jobDict[ 'MaxJobs' ] = maxJobs
        jobDict[ 'JobUsage' ] = math.floor(float(jobDict[ 'Running' ]) / maxJobs * 1000) / 10
      else:
        jobDict[ 'MaxJobs' ] = ''
        jobDict[ 'JobUsage' ] = ''

    return S_OK(sitesJobSummary)

  types_getSitesWNSummary = [ (str, NoneType, list) ]
  def export_getSitesWNSummary(self, sitesName):
    """
    Return the dictionary with work node summary information for the given sites.

    :return: S_OK( { site : { 'WNStatus' :  'OK'
                                                   } } ) | S_ERROR
    """

    gLogger.info('getSitesWNSummary')

    sitesWNSummary = {}

    queryRes = rmClient.selectWorkNodeCache(site = sitesName)
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]
    columns = queryRes[ 'Columns' ]

    wnsEfficiency = {}
    for record in records:
      recordDict = dict( zip( columns, record ) )
      site = recordDict[ 'Site' ]
      if site not in wnsEfficiency:
        wnsEfficiency[ site ] = []
      wnsEfficiency[site ].append( recordDict[ 'Efficiency' ] )

    for site, efficiencyList in wnsEfficiency.items():
      if len(efficiencyList) != 0:
        rate = float( efficiencyList.count( 0 ) ) / len( efficiencyList )
        if rate == 0:
          status = 'OK'
        elif rate > 0.5:
          status = 'Bad'
        else:
          status = 'Warn'
        sitesWNSummary[ site ] = { 'WNStatus' : status }

    return S_OK( sitesWNSummary )

  types_getSiteWNsInfo = [ str ]
  def export_getSiteWNsInfo(self, siteName):
    """
    Retruns the jobs statistics for hosts of the given site.

    :return: S_OK( [ { 'Host' : 'aaa.bb.ccc'
                                        'Running' : 1
                                        'Done' : 22
                                        'Failed' : 22
                                        'Efficiency' : 50.0
                                        } ] ) / S_ERROR
    """

    gLogger.info('getSiteWNsInfo')

    queryRes = rmClient.selectWorkNodeCache(site = siteName,
                                       meta = { 'columns' : [ 'Host', 'Done', 'Failed', 'Efficiency' ] })
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]
    columns = queryRes[ 'Columns' ]

    results = []
    for record in records:
      results.append(dict(zip( columns, record )))

    return S_OK(results)

  types_getSAMDetail = [ str, str ]
  def export_getSAMDetail(self, elementName, testType):
    """
    Returns the dictionary with SAM test detail information for the given test.
    """

    gLogger.info('getSAMDetail')

    queryRes = rmClient.selectSAMResult(elementName=elementName, testType=testType)
    if not queryRes[ 'OK' ]:
      return queryRes
    record = queryRes[ 'Value' ][ 0 ]
    columns = queryRes[ 'Columns' ]

    detail = dict(zip(columns, record))
    detail.pop('LastCheckTime')
    return S_OK(detail)

  types_getSAMSummary = [ str, str ]
  def export_getSAMSummary(self, siteName, vo):
    """
    Returns SAM tests status for the elements of the given site.

    :return: S_OK( { element : { 'ElementType' :
                                                            'WMSTest' :
                                                            'CVMFSTest' :
                                                            'BOSSTest' :
                                                            'SETest' : } } ) / S_ERROR

    """

    gLogger.info('getSAMSummary')

    siteType = siteName.split('.')[ 0 ]
    if 'CLOUD' == siteType:
      ces = [ siteName ]
    else:
      ces = CSHelpers.getSiteComputingElements(siteName)
    ses = CSHelpers.getSiteStorageElements(siteName)

    samSummary = {}
    for ce in ces:
      samSummary[ ce ] = { 'ElementType' : 'ComputingElement' }
    for se in ses:
      samSummary[ se ] = { 'ElementType' : 'StorageElement' }

    lastCheckTime = datetime.utcnow().replace(microsecond = 0) - timedelta(hours = 24)

    queryRes = rmClient.selectResourceSAMStatus(elementName = ces, vO = vo,
                                                meta = { 'newer' : [ 'LastCheckTime', lastCheckTime ] })
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]
    columns = queryRes[ 'Columns' ]

    if ses != []:
      queryRes = rmClient.selectResourceSAMStatus(elementName = ses,
                                                  meta = { 'newer' : [ 'LastCheckTime', lastCheckTime ] })
      if not queryRes[ 'OK' ]:
        return queryRes
      records += queryRes[ 'Value' ]

    for record in records:
      samDict = dict(zip(columns, record))
      elementName = samDict[ 'ElementName' ]
      samSummary[ elementName ][ 'Status' ] = samDict[ 'Status' ]
      tests = [ test.strip() for test in samDict[ 'Tests' ].split(',') ]
      queryRes = rmClient.selectSAMResult(elementName = elementName, testType = tests,
                                          meta = { 'newer' : [ 'LastCheckTime', lastCheckTime ] })
      if not queryRes[ 'OK' ]:
        return queryRes
      testRecords = queryRes[ 'Value' ]
      testColumns = queryRes[ 'Columns' ]
      for testRecord in testRecords:
        testDict = dict(zip(testColumns, testRecord))
        samSummary[ elementName ][ testDict[ 'TestType' ] ] = testDict[ 'Status' ]

    return S_OK(samSummary)

  def __getSitesMaxJobs(self, sites):
    sitesMaxJobs = {}

    _basePath = 'Resources/Sites'

    for site in sites:
      domain = site.split('.')[ 0 ]

      if domain == 'CLOUD':
        maxJobs = 0
        endpoints = gConfig.getSections('%s/%s/%s/Cloud' % (_basePath, domain, site))
        if not endpoints[ 'OK' ]:
          gLogger.warn(endpoints['Message'])
          continue
        endpoints = endpoints[ 'Value' ]
        for endpoint in endpoints:
          maxJobs += gConfig.getValue('%s/%s/%s/Cloud/%s/MaxInstances' % (_basePath, domain, site, endpoint), 0)
        sitesMaxJobs[ site ] = maxJobs

      else:
        maxJobs = 0
        ces = gConfig.getSections('%s/%s/%s/CEs' % (_basePath, domain, site))
        if not ces[ 'OK' ]:
          return ces
        ces = ces[ 'Value' ]
        for ce in ces:
          queues = gConfig.getSections('%s/%s/%s/CEs/%s/Queues' % (_basePath, domain, site, ce))
          if not queues[ 'OK' ]:
            gLogger.warn(queues['Message'])
            continue
          queues = queues[ 'Value' ]
          for queue in queues:
            maxJobs += gConfig.getValue('%s/%s/%s/CEs/%s/Queues/%s/MaxTotalJobs' % (_basePath, domain, site, ce, queue), 0)
        sitesMaxJobs[ site ] = maxJobs

    return S_OK(sitesMaxJobs)

  types_getTestHistory = [ str, str, datetime, datetime]
  def export_getTestHistory(self, elementType, element, fromDate, toDate):
    gLogger.info('getTestHistory')

    if fromDate > toDate:
      return S_ERROR('from date can not be after the to date.')

    selectElements = []
    if elementType == 'Site':
      if element.split('.')[ 0 ] == 'CLOUD':
        selectElements.append( element )
      else:
        selectElements += CSHelpers.getSiteComputingElements(element)
      selectElements += CSHelpers.getSiteStorageElements(element)
    else:
      selectElements = [ element ]

    queryRes = rmClient.selectSAMResultLog(
                                           elementName = selectElements,
                                           meta = { 'newer' : ['LastCheckTime', fromDate ],
                                                   'older' : [ 'LastCheckTime', toDate ],
                                                   'columns' : [ 'ElementName', 'TestType', 'Status', 'LastCheckTime' ] }
                                           )
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]

    testHistory = {}
    for record in records:
      key = record[ 0 ] + '-' + record[ 1 ]
      if key not in testHistory:
        testHistory[ key ] = []
      testHistory[ key ].append(( record[ 3 ], record[ 2 ] ))

    return S_OK(testHistory)

  types_getResourceStatusHistory = [ (str, NoneType, list), str, datetime, datetime ]
  def export_getResourceStatusHistory(self, resources, vo, fromDate, toDate):
    gLogger.info('getResourceStatusHistory')

    if fromDate > toDate:
      return S_ERROR('from date can not be after the to date.')

    queryRes = rmClient.selectResourceSAMStatusLog(
                                                   elementName = resources, vO = vo,
                                                   meta = { 'newer' : ['LastCheckTime', fromDate ],
                                                           'older' : [ 'LastCheckTime', toDate ],
                                                           'columns' : [ 'ElementName', 'Status', 'LastCheckTime' ] }
                                                   )
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]

    statusHistory = {}
    for record in records:
      res = record[ 0 ]
      if res not in statusHistory:
        statusHistory[ res ] = []
      statusHistory[ res ].append(( record[ 2 ], record[ 1 ] ))

    return S_OK(statusHistory)

  types_getSiteStatusHistory = [ (str, NoneType, list), str, datetime, datetime ]
  def export_getSiteStatusHistory(self, sites, vo, fromDate, toDate):
    gLogger.info('getSiteStatusHistory')

    if fromDate > toDate:
      return S_ERROR('from date can not be after the to date.')

    queryRes = rmClient.selectSiteSAMStatusLog(
                                               site = sites, vO = vo,
                                               meta = { 'newer' : ['LastCheckTime', fromDate ],
                                                       'older' : [ 'LastCheckTime', toDate ],
                                                       'columns' : [ 'Site', 'Status', 'LastCheckTime' ] }
                                                   )
    if not queryRes[ 'OK' ]:
      return queryRes
    records = queryRes[ 'Value' ]

    statusHistory = {}
    for record in records:
      site = record[ 0 ]
      if site not in statusHistory:
        statusHistory[ site ] = []
      statusHistory[ site ].append(( record[ 2 ], record[ 1 ] ))

    return S_OK(statusHistory)

# ...............................................................................
# EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF



""" WMSTest

  A simplest test to test job submission function.

"""

from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.CEBaseTest import CEBaseTest


__RCSID__ = '$Id: $'


class WMSTest( CEBaseTest ):
  """
    WMSTest is used to test whether jobs can be submitted to the specified
    ce or cloud.
  """

  def __init__( self, args = None, apis = None ):
    super( WMSTest, self ).__init__( args, apis )


  @staticmethod
  def _judge( log ):
    """
      judge the WMS test status.
    """

    if log.find( 'hello' ) != -1:
      return 'OK'
    else:
      return 'Bad'



from DIRAC.ResourceStatusSystem.Policy.Configurations import POLICIESMETA as DIRACPOLICIESMETA

__RCSID__ = '$Id:  $'

POLICIESMETA = DIRACPOLICIESMETA

BESPOLICIESMETA = {
    'SiteSAM' : {
        'description' : 'Policy based on site SAM information',
        'module' : 'SiteSAMPolicy',
        'command' : ( 'SAMCommand', 'SAMCommand' ),
        'args' : None
    }
}

POLICIESMETA.update( BESPOLICIESMETA )



import math
from datetime import datetime, timedelta
from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen
from DIRAC.Core.DISET.RPCClient import RPCClient



class SAMHistoryHandler(WebHandler):
    
  AUTH_PROPS = "authenticated"
        
  @asyncGen
  def web_getPlotData(self):
    publisher = RPCClient('ResourceStatus/PublisherIHEP')
    
    args = self._request()
    category = args[ 'Category' ]
    elementType = args[ 'ElementType' ]
    if 'Elements' not in args:
      elements = yield self.threadTask(self.__getElementsByType, elementType, publisher)
    else:
      elements = args[ 'Elements' ]
    vo = args.get( 'VO' ) or 'all'
    fromDatetime = args[ 'From' ].replace(minute = 0)
    toDatetime = args[ 'To' ].replace(minute = 0)
    
    if category == 'TestResults':
      testHistory = yield self.threadTask(publisher.getTestHistory, elementType, elements[ 0 ], fromDatetime, toDatetime)
      if not testHistory[ 'OK' ]:
        self.finish({ 'success' : 'false', 'result' : [], 'error' : testHistory[ 'Message' ] })
        return
      testHistory = testHistory[ 'Value' ]
      if not testHistory:
        self.finish({ 'success' : 'false', 'result' : [], 'error' : 'There are no data to display.' })
        return
      data, elements, timestamps = self.__generDetailData(testHistory, fromDatetime, toDatetime)
      self.finish({ 'success' : 'true', 'result' : data, 'elements' : elements, 'timestamps' : timestamps })
    
    else:
      if elementType == 'Site':
        statusHistory = yield self.threadTask(publisher.getSiteStatusHistory, elements, vo, fromDatetime, toDatetime)
      else:
        statusHistory = yield self.threadTask(publisher.getResourceStatusHistory, elements, vo, fromDatetime, toDatetime)
      if not statusHistory[ 'OK' ]:
        self.finish({ 'success' : 'false', 'result' : [], 'error' : statusHistory[ 'Message' ] })
        return
      statusHistory = statusHistory[ 'Value' ]
      if not statusHistory:
        self.finish({ 'success' : 'false', 'result' : [], 'error' : 'There are no data to display.' })
        return
    
      if category == 'Availability':
        data, elements = self.__generSummaryData(statusHistory)
        self.finish({ 'success' : 'true', 'result' : data, 'elements' : elements })
      else:
        data, elements, timestamps = self.__generDetailData(statusHistory, fromDatetime, toDatetime)
        self.finish({'success' : 'true', 'result' : data, 'elements' : elements, 'timestamps' : timestamps })


  @asyncGen
  def web_getElements(self):
    publisher = RPCClient('ResourceStatus/PublisherIHEP')
    
    elementType = self._request()[ 'ElementType' ]
    elements = yield self.threadTask(self.__getElementsByType, elementType, publisher)
      
    data = []
    for el in elements:
      data.append(dict( value = el, text = el ))
      
    self.finish({ 'success' : 'true', 'result' : data })
    

  @asyncGen
  def web_getVOs(self):
    publisher = RPCClient('ResourceStatus/PublisherIHEP')
    
    vos = yield self.threadTask( publisher.getVOs )
    if vos[ 'OK' ]:
      vos = vos[ 'Value' ]
    else:
     self.finish( { 'success' : 'false', 'error' : vos[ 'Message' ] } )
      
    defaultVO = yield self.threadTask( publisher.getVOByGroup, self.getSessionData()[ 'user' ][ 'group' ] )
    defaultVO = defaultVO[ 'Value' ]
    
    self.finish( { 'success' : 'true', 'result' : { 'VOs' : vos, 'defaultVO' : defaultVO } } )


  def __getElementsByType(self, elementType, publisher):
    elements = []
    if 'Site' == elementType:
      elements = publisher.getSites()[ 'Value' ]
    elif 'ComputingElement' == elementType:
      elements = publisher.getComputingElements()[ 'Value' ]
      elements += publisher.getDomainSites('CLOUD')[ 'Value' ]
    elif 'StorageElement' == elementType:
      elements = publisher.getStorageElements()[ 'Value' ]
    return elements


  def __generSummaryData(self, historyData):
    summaryData = []
    
    for element, historyList in historyData.items():
      okNum = 0
      for date, status in historyList:
        if status == 'OK' or status == 'Busy':
          okNum += 1
      okRate = math.floor((float(okNum) / len(historyList)) * 10000) / 100
      summaryData.append(( element, okRate ))
      
    summaryData.sort(key = lambda e : e[ 0 ])
    elements = [ e[0] for e in summaryData ]
    data = [ e[ 1 ] for e in summaryData ]
    return data, elements
        
        
  def __generDetailData(self, historyData, fromDate, toDate):
    detailData = []
    
    interval = self.__getInterval(fromDate, toDate)
    timestamps = self.__getTimeStampsByInterval(fromDate, toDate, interval)
    dateFormat = self.__getDateFormatByInterval(interval)
    
    for element, historyList in historyData.items():
      stampIdx = 0
      showStatuses = []
      oriStatuses= []
      for date, status in historyList:
        while(stampIdx < len(timestamps) - 1 and date >= timestamps[ stampIdx + 1 ]):
          showStatuses.append(self.__combine(oriStatuses))
          stampIdx += 1
          oriStatuses = []
        oriStatuses.append(status)
      while(stampIdx < len(timestamps)):
        showStatuses.append(self.__combine(oriStatuses))
        stampIdx += 1
        oriStatuses = []
      detailData.append(( element, showStatuses ))
      
    detailData.sort(key = lambda e : e[ 0 ])
    elements = [ e[ 0 ] for e in detailData ]
    data = [ e[ 1 ] for e in detailData ]
    timestamps = map(lambda arg : arg.strftime(dateFormat), timestamps)
    return data, elements, timestamps

    
  def __combine(self, statuses):
    if len(statuses) == 0:
      return None
    if float(statuses.count('Unknown')) / len(statuses) > 0.9:
      return  -1
    if float(statuses.count('Busy')) / len(statuses) > 0.9:
      return  -2
    return math.floor(float(statuses.count('OK') + statuses.count('Busy')) / len(statuses) * 100) / 100


  def __getInterval(self, fromDate, toDate):
    interval = toDate - fromDate
    if interval > timedelta(days = 210):
      return 'month'
    if interval >timedelta(days = 7):
      return 'day'
    return 'hour'


  def __getDateFormatByInterval(self, interval):
    if 'month' == interval:
      return '%Y-%m'
    if 'day' == interval:
      return '%m-%d'
    if 'hour' == interval:
      return '%m-%d %H:00'
  
  
  def __getTimeStampsByInterval( self, fromDate, toDate, interval ):
    timestamps = []
    
    if 'month' == interval:
      fromY = fromDate.year
      fromM = fromDate.month
      stamp = datetime.strptime('%d-%d' % (fromY, fromM), '%Y-%m')
      while stamp <= toDate:
        timestamps.append(stamp)
        fromY += fromM / 12
        fromM = (fromM % 12) + 1
        stamp = datetime.strptime('%d-%d' % (fromY, fromM), '%Y-%m')
        
    elif 'day' == interval:
      stamp = datetime.strptime(fromDate.strftime('%Y-%m-%d'), '%Y-%m-%d')
      while stamp <= toDate:
        timestamps.append(stamp)
        stamp += timedelta(days = 1)
        
    elif 'hour' == interval:
      stamp = datetime.strptime(fromDate.strftime('%Y-%m-%d %H'), '%Y-%m-%d %H')
      while stamp < toDate:
        timestamps.append(stamp)
        stamp += timedelta(hours = 1)
    
    return timestamps

    
  def _request(self):
    args = {}
    
    if 'category' in self.request.arguments:
      args[ 'Category' ] = self.request.arguments[ 'category' ][ 0 ]
      
    if 'elementType' in self.request.arguments:
      args[ 'ElementType' ] = self.request.arguments[ 'elementType' ][ 0 ]
      
    if 'elements' in self.request.arguments:
      args[ 'Elements' ] = self.request.arguments[ 'elements' ]

    if 'vo' in self.request.arguments and self.request.arguments[ 'vo' ][ 0 ] != '' :
      args[ 'VO' ] = self.request.arguments[ 'vo' ][ 0 ]

    if 'from' in self.request.arguments:
      args[ 'From' ] = datetime.strptime(self.request.arguments[ 'from' ][ 0 ], '%Y-%m-%d %H:%M')
      
    if 'to' in self.request.arguments:
      args[ 'To' ] = datetime.strptime(self.request.arguments[ 'to' ][ 0 ], '%Y-%m-%d %H:%M')
    
    return args




TESTS = {

  'Access-Test' :
  {
    'module' : 'AccessTest',
    'match' : { 'ElementType' : ( 'ComputingElement', 'CLOUD', 'StorageElement' ) },
    'args' : { 'timeout' : 5 }
  },

  'WMS-Test' :
  {
    'module' : 'WMSTest',
    'match' : { 'ElementType' : ( 'ComputingElement', 'CLOUD' ) },
    'args' : { 'executable' : 'wms_test.py', 'timeout' : 2400 }
  },

  'CVMFS-Test' :
  {
    'module' : 'CVMFSTest',
    'match' : { 'ElementType' : ( 'ComputingElement', 'CLOUD' ), 'VO' : 'bes' },
    'args' : { 'executable' : 'cvmfs_test.py', 'timeout' : 2400 }
  },

  'BOSS-Test' :
  {
    'module' : 'BOSSTest',
    'match' : { 'ElementType' : ( 'ComputingElement', 'CLOUD' ), 'VO' : 'bes' },
    'args' : { 'executable' : 'boss_test.py', 'timeout' : 2400 }
  },

  'CEPC-Test' :
  {
    'module' : 'CEPCTest',
    'match' : { 'ElementType' : ( 'ComputingElement', 'CLOUD' ), 'VO' : 'cepc' },
    'args' : { 'executable' : 'cepc_test.py', 'timeout' : 2400 }
  },

  'JUNO-Test':
  {
   'module' : 'JUNOTest',
   'match' : { 'ElementType' : ( 'ComputingElement', 'CLOUD' ), 'VO' : 'juno' },
   'args' : { 'executable' : 'juno_test.py', 'timeout' : 2400 }
  },

  'SE-Test' :
  {
    'module' : 'SETest',
    'match' : { 'ElementType' : ( 'StorageElement', ) }
  },

}



""" StorageIHEPCommand

  The StorageIHEPCommand class is a command class to know about the storage capacity.

"""

import math
from DIRAC                                                         import S_OK, gConfig
from DIRAC.DataManagementSystem.DB.FileCatalogDB                   import FileCatalogDB
from DIRAC.ResourceStatusSystem.Command.Command                    import Command
from IHEPDIRAC.ResourceStatusSystem.Utilities                      import CSHelpers
from IHEPDIRAC.ResourceStatusSystem.Client.ResourceManagementIHEPClient import ResourceManagementIHEPClient



class StorageIHEPCommand( Command ):
  """
    StorageIHEPCommand
  """

  def __init__( self, args = None, clients = None ):
    super( StorageIHEPCommand, self ).__init__( args, clients )

    if 'ResourceManagementIHEPClient' in self.apis:
      self.rmIHEPClient = self.apis[ 'ResourceManagementIHEPClient' ]
    else:
      self.rmIHEPClient = ResourceManagementIHEPClient()

    if 'FileCatalogDB' in self.apis:
      self.fcDB = self.apis[ 'FileCatalogDB' ]
    else:
      self.fcDB = FileCatalogDB()

  def _storeCommand( self, result):
    """
      Stores the results of doNew method on the database.
    """

    for storageDict in result:
      resQuery = self.rmIHEPClient.addOrModifyStorageCache( sE = storageDict[ 'SE' ],
                                                        occupied = storageDict[ 'Occupied' ],
                                                        free = storageDict[ 'Free' ],
                                                        usage = storageDict[ 'Usage' ] )
      if not resQuery[ 'OK' ]:
        return resQuery
    return S_OK()


  def doNew( self, masterParams = None ):
    """
      It searches FileCatalogDB to find out occupied storage.
    """

    ses = masterParams

    seMaxStorage = {}
    for se in ses:
      maxStorage = gConfig.getValue('/Resources/StorageElements/%s/Capacity' % se, 0) * 1 << 40
      seMaxStorage[ se ] = maxStorage

    sqlStr = """select SE.SEName, sum(F.Size) from
    FC_Replicas R, FC_Files F, FC_StorageElements SE
    where R.FileID=F.FileID and R.SEID=SE.SEID
    group by R.SEID;"""

    result = self.fcDB._query(sqlStr)
    if not result[ 'OK' ]:
      return result
    result = result[ 'Value' ]

    seOccupied = {}
    for se, occupied in result:
      seOccupied[ se ] = int(occupied)

    uniformResult = []
    for se in ses:
      max = seMaxStorage.get(se, 0)
      occupied = seOccupied.get(se, 0)
      if max == 0:
        usage = 0.0
        free = 0
      else:
        usage = math.floor(float(occupied) / max * 1000) / 10
        free = max - occupied
      uniformResult.append( { 'SE' : se, 'Occupied' : occupied, 'Free' : free, 'Usage' : usage } )

    storeRes = self._storeCommand( uniformResult )
    if not storeRes[ 'OK' ]:
      return storeRes

    return S_OK( result )


  def doMaster(self):
    """
      Master method

      Gets all ses and call doNew method
    """

    ses = CSHelpers.getStorageElements()
    if not ses[ 'OK' ]:
      return ses

    storageResults = self.doNew( ses[ 'Value' ] )
    if not storageResults[ 'OK' ]:
      self.metrics[ 'failed' ].append( storageResults[ 'Message' ] )

    return S_OK( self.metrics )



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from setuptools import setup

# This is required to allow editable pip installs while using the declarative configuration (setup.cfg)
setup()




''' AccessTest

The base access test class to test the access to ces, clouds and ses.

'''

import subprocess, re
from datetime import datetime
from DIRAC import S_OK, S_ERROR, gConfig
from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.TestBase import TestBase
from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.CEAccessTest import CEAccessTest
from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.CLOUDAccessTest import CLOUDAccessTest
from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.SEAccessTest import SEAccessTest


class AccessTest( TestBase ):
  ''' AccessTest
  '''

  def __init__(self, args=None, apis=None):
    super( AccessTest, self ).__init__( args, apis )

    self.timeout = self.args.get( 'timeout', 5 )
    self.ce = CEAccessTest()
    self.cloud = CLOUDAccessTest()
    self.se = SEAccessTest()

  def doTest(self, elementDict):
    '''
      Use nc tool to test the access to the specified element.
      Test command is like 'nc -v -w 5 -z 202.122.33.148 22'.
    '''

    elementName = elementDict[ 'ElementName' ]
    elementType = elementDict[ 'ElementType' ]

    params = self._getAccessParams( elementName, elementType )
    if not params[ 'OK' ]:
      return params
    host, port = params[ 'Value' ]
    
    command = 'nc -v -w %d -z %s %s' % ( self.timeout, host, port )
    print "Access tests command:", command
    submissionTime = datetime.utcnow().replace( microsecond = 0 )
    subp = subprocess.Popen( command, shell = True, stdout = subprocess.PIPE, stderr = subprocess.PIPE )
    stdout, stderr = subp.communicate()
    completionTime = datetime.utcnow().replace( microsecond = 0 )
    applicationTime = (completionTime - submissionTime).total_seconds()

    if subp.returncode == 0:
      status = 'OK'
      log = stdout
    else:
      status = 'Bad'
      log = stderr

    result = { 'Result' : { 'Status' : status,
                           'Log' : log,
                           'SubmissionTime' : submissionTime,
                           'CompletionTime' : completionTime,
                           'ApplicationTime' : applicationTime },
              'Finish' : True }

    return S_OK(result)


  def _getAccessParams( self, elementName, elementType ):
    '''
      get the access host and port for the specified element.
    '''

    if elementType == 'ComputingElement':
      return self.ce._getAccessParams( elementName )
    elif elementType == "CLOUD":
      return self.cloud._getAccessParams( elementName )
    elif elementType == "StorageElement":
      return self.se._getAccessParams( elementName )
    else:
      return S_ERROR( "The elementType is not vaild." )



import time
import subprocess

from DIRAC import gConfig, gLogger

from IHEPDIRAC.Badger.private.output.getfile.GetFile import GetFile
from IHEPDIRAC.Badger.private.output.getfile.Rsync   import Rsync

class DaemonRsyncGetFile(Rsync, GetFile):
  def __init__(self):
    super(DaemonRsyncGetFile, self).__init__()

    self.__rsyncUrl = gConfig.getValue('/Resources/Applications/DataLocation/RsyncEndpoints/Url', 'rsync://localhost/bes-srm')
    gLogger.debug('Rsync daemon url:', self.__rsyncUrl)

  def _available(self):
    args = ['rsync', self.__rsyncUrl]
    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = p.communicate()
    ret = p.returncode

    return ret == 0 and out

  def _lfnToRemote(self, lfn):
    return self.__rsyncUrl + lfn



#!/usr/bin/env python

import zlib

class Adler32CheckSum():
  @staticmethod
  def checksum(filename, blocksize=1048576):
    value = 1
    with open(filename, "rb") as f:
      for block in iter(lambda: f.read(blocksize), ""):
        value = zlib.adler32(block, value)
    return hex(value & 0xffffffff).lower().replace('l','').replace('x','0')[-8:]
